--[[	GAOA - Guild Applicant Online Alert
	by SDPhantom
	http://www.phantomweb.org	]]
------------------------------------------

local name,addon=...;
addon.Options={
	PingTimeout=30;--	Timeout for offline detection (Applicant is online if this is reached)
	UpdateThrottle=5;--	Throttles timeout and reping checks
	RepingOffline=15;--	Timeout for repinging offline applicants
}
GAOA_Options=addon.Options;

--------------------------
--[[	Main Code	]]
--------------------------
--	Data is stored as {LastPing,Pinging,Online}
local status={};
local pingqueue={};
local lastupdate=0;
local offlinepat="^"..ERR_CHAT_PLAYER_NOT_FOUND_S:gsub("%%%d?%$?s","(.-)").."$";
local guilduiloaded=false;
local eframe=CreateFrame("Frame");
eframe:RegisterEvent("ADDON_LOADED");
eframe:RegisterEvent("PLAYER_LOGIN");
eframe:RegisterEvent("LF_GUILD_RECRUIT_LIST_CHANGED");
eframe:RegisterEvent("LF_GUILD_RECRUITS_UPDATED");

eframe:SetScript("OnEvent",function(self,event,...)
	if event=="ADDON_LOADED" and (...)==name then
		addon.Options=GAOA_Options;
	elseif event=="PLAYER_LOGIN" or event=="LF_GUILD_RECRUIT_LIST_CHANGED" then
		RequestGuildApplicantsList();
	elseif event=="LF_GUILD_RECRUITS_UPDATED" then
		local oldlist=status;status={};
		local now=GetTime();
		local numapp=GetNumGuildApplicants();
		for i=1,numapp do
			local name=GetGuildApplicantInfo(i);

			ptr=oldlist[name] or {};
			ptr[1]=now;
			ptr[2]=true;
			ptr[3]=ptr[3] or false;

			status[name]=ptr;
			table.insert(pingqueue,name);
			if guilduiloaded then GuildInfoFrameApplicants_Update(); end
		end
	end
end);

eframe:SetScript("OnUpdate",function(self)
	if #pingqueue>0 then
		SendAddonMessage("GAOA_Ping","","WHISPER",pingqueue[1]);
		table.remove(pingqueue,1);
	end

	local now=GetTime();
	if now>lastupdate+addon.Options.UpdateThrottle then
		for i,j in pairs(status) do
			if j[2] and now>j[1]+addon.Options.PingTimeout then
				if not j[3] then
					DEFAULT_CHAT_FRAME:AddMessage("GAOA: "..ERR_FRIEND_ONLINE_SS:format(i,i),0.5,1,0);
					PlaySoundKitID(18019);-- BNet toast popup sound
				end

				j[2]=false;
				j[3]=true;

				if IsAddOnLoaded("Blizzard_GuildUI") then GuildInfoFrameApplicants_Update(); end
			elseif j[3] or (not j[2] and now>j[1]+addon.Options.RepingOffline) then
				j[1]=now;
				j[2]=true;

				table.insert(pingqueue,i);
				if guilduiloaded then GuildInfoFrameApplicants_Update(); end
			end
		end
		lastupdate=now;
	end
end);

ChatFrame_AddMessageEventFilter("CHAT_MSG_SYSTEM",function(self,event,msg)
	local name=msg:match(offlinepat);
	if name and status[name] and GetTime()<=status[name][1]+30 then
		local ptr=status[name];
		if ptr[3] then
			DEFAULT_CHAT_FRAME:AddMessage("GAOA: "..ERR_FRIEND_OFFLINE_S:format(name),0.5,1,0);
			PlaySoundKitID(18019);-- BNet toast popup sound
		end

		ptr[2]=false;
		ptr[3]=false;

		if guilduiloaded then GuildInfoFrameApplicants_Update(); end
		return true;
	end
end);

----------------------------------
--[[	Blizzard_GuildUI Mod	]]
----------------------------------
function HookButtonUpdate()
	hooksecurefunc("GuildInfoFrameApplicants_Update",function()
		local frame=GuildInfoFrameApplicantsContainer;
		local off=HybridScrollFrame_GetOffset(frame);
		for i,j in ipairs(frame.buttons) do
			local name=GetGuildApplicantInfo(off+i);
			local ptr=status[name];
			if ptr then
				j.name:SetFormattedText("%s%s"
					,("|TInterface\\FriendsFrame\\StatusIcon-%s:0|t"):format(
						ptr[3] and "Online"
					or	ptr[2] and "Away"
					or	"Offline"
					)
					,name
				);
			end
		end

		local num=GetNumGuildApplicants();
		if num>0 then
			local cnt=0;
			for i,j in pairs(status) do
				if j[3] then
					cnt=cnt+1;
				end
			end
			if cnt>0 then
				GuildInfoFrameTab3:SetFormattedText(GUILDINFOTAB_APPLICANTS:gsub("%%d","|cff00ff00%%d|r/%%d"),cnt,num);
			end
		end
	end);

--	Fix for scrolling problem
	GuildInfoFrameApplicantsContainer.update=GuildInfoFrameApplicants_Update;

	guilduiloaded=true;
end

if IsAddOnLoaded("Blizzard_GuildUI") then
	HookButtonUpdate();
else
	eframe:HookScript("OnEvent",function(self,event,...)
		if event=="ADDON_LOADED" and (...)=="Blizzard_GuildUI" then
			HookButtonUpdate();
		end
	end);
end
